#include <iostream>
#include <fstream>
#include <sstream>
#include <list>
#include <time.h>
#include <dirent.h>
#include <stdio.h>
#include "opencv2/opencv.hpp"
#include "opencv2/core.hpp"
#include "opencv2/highgui.hpp"
#include "opencv2/imgproc.hpp"
#include "opencv2/imgcodecs.hpp"
#include "opencv2/videoio.hpp"
#include "libavformat/avformat.h"
#include "opencv2/face.hpp"
#include "opencv2/objdetect.hpp"
#include "opencv2/video.hpp"
#include "opencv2/optflow.hpp"
#include "metric.h"

using namespace cv;
using namespace cv::face;
using namespace std;

float sigmac            = 64;
int histogramSize1D     = 16;

// DEFAULT PARAMETERS //
string dataPath = "./input.avi";
string csvPath = "no_csv_data";
string outputType = "0";

int histogramSize2D     = histogramSize1D * histogramSize1D;
int histogramSize3D     = histogramSize2D * histogramSize1D;
int logSize             = (int)log2(histogramSize1D);
int logSize2            = 2*logSize;
float fps = 24;

Mat squares             = Mat::zeros(1, 10000, CV_32FC1);
float* squaresPtr       = squares.ptr<float>(0);
vector<Mat> LAB;
vector<float> L, A, B;
float meanVectorFloat[4] = {0.5555,    0.6449,    0.0002,    0.0063};
float inverseCovarianceFloat[4][4] = {{43.3777,    1.7633,   -0.4059,    1.0997},
                                      {1.7633,   40.7221,   -0.0165,    0.0447},
                                      {-0.4059,   -0.0165,   87.0455,   -3.2744},
                                      {1.0997,    0.0447,   -3.2744,  125.1503}};

Mat modelMean                   = Mat(4, 1, CV_32FC1, meanVectorFloat);
Mat modelInverseCovariance      = Mat(4, 4, CV_32FC1, inverseCovarianceFloat);

Metric M;

VideoWriter outputVideo;

// METRICS //

double MSE = 0, CCORR = 0, LCC = 0, KLD = 0, J = 0;


// WEIGHTS //

float w_globalContrast = 0.25;
float w_gaussianDist = 0.25;
float w_face = 0.25;
float w_movement = 0.25;

// READ FILES //

void readFiles(vector<string>& imagePaths,
               vector<string>& imageNames,
               VideoCapture &cap){

    // VIDEO //

    cap        = VideoCapture(dataPath.c_str());
    if(!cap.isOpened()){
        cout << "Video cannot be opened!" << endl;
        return;
    }
    string::size_type pAt = dataPath.find_last_of('.');
    const string NAME = dataPath.substr(0, pAt) + "_output" + ".avi";
    fps = cap.get(CAP_PROP_FPS);
    int fourcc = cv::VideoWriter::fourcc('X', 'V', 'I', 'D');
    Size S = Size((int) cap.get(CV_CAP_PROP_FRAME_WIDTH), (int) cap.get(CV_CAP_PROP_FRAME_HEIGHT));
    for (int i = 0; i < cap.get(CV_CAP_PROP_FRAME_COUNT); i++) {
        string temp = to_string(i) + ".jpg";
        imageNames.push_back(temp);
    }
    cout << "Opening video: " << NAME << endl;
    outputVideo = VideoWriter(NAME, fourcc, fps, S);
    if(!outputVideo.isOpened()){
        cout << "Output video cannot be opened!" << endl;
        return;
    }
    sort(imageNames.begin(), imageNames.end());
    for (const auto& imName : imageNames) {
        imagePaths.push_back( dataPath + imName );
    }
}


Mat globalContrast(Mat im){
    Mat colorDistance = Mat(im.rows, im.cols, im.type(), Scalar(0,0,0));
    Mat distFromCenter = Mat(im.rows, im.cols, im.type(), Scalar(0,0,0));
    cvtColor(im,im,cv::COLOR_BGR2Lab);

    //COLORS REDUCTION K-MEANS
    Mat samples(im.rows * im.cols, 3, CV_32F);
    for( int y = 0; y < im.rows; y++ )
        for( int x = 0; x < im.cols; x++ )
            for( int z = 0; z < 3; z++)
                samples.at<float>(y + x*im.rows, z) = im.at<Vec3b>(y,x)[z];

    int clusterCount = 2;
    int attempts = 5;
    Mat labels;
    Mat centers;
    kmeans(samples, clusterCount, labels, TermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS, 10000, 0.0001), attempts, KMEANS_PP_CENTERS, centers);

    //imshow("", centers);
    //waitKey(0);

    int histogram[clusterCount];
    Vec3b clusterValue[clusterCount];
    float colorDist[clusterCount];

    Point colorCentersMinMax[clusterCount][2]; // [0] = min, [1] = max
    Point colorCenters[clusterCount];

    for (int i=0; i<clusterCount; i++){
        histogram[i] = 0;
        colorDist[i] = 0.0;
        colorCentersMinMax[i][0] = Point(im.cols/2,im.rows/2);
        colorCentersMinMax[i][1] = Point(im.cols/2,im.rows/2);
    }

    for( int y = 0; y < im.rows; y++ ){
        for( int x = 0; x < im.cols; x++ )
        {
            int cluster_idx = labels.at<int>(y + x*im.rows,0);
            im.at<Vec3b>(y,x)[0] = centers.at<float>(cluster_idx, 0);
            im.at<Vec3b>(y,x)[1] = centers.at<float>(cluster_idx, 1);
            im.at<Vec3b>(y,x)[2] = centers.at<float>(cluster_idx, 2);
            clusterValue[cluster_idx] = im.at<Vec3b>(y,x);
            histogram[cluster_idx]++;

            if (colorCentersMinMax[cluster_idx][0].x > x) colorCentersMinMax[cluster_idx][0].x = x;
            if (colorCentersMinMax[cluster_idx][0].y > y) colorCentersMinMax[cluster_idx][0].y = y;
            if (colorCentersMinMax[cluster_idx][1].x < x) colorCentersMinMax[cluster_idx][1].x = x;
            if (colorCentersMinMax[cluster_idx][1].y < y) colorCentersMinMax[cluster_idx][1].y = y;

        }
    }

    float diff;
    float min = 255;
    float max = 0;

    for (int i=0; i<clusterCount; i++){

        //COLOR DISTANCE OF REDUCED COLORS
        diff = 0;
        for (int j=0; j<clusterCount; j++){
            Scalar R = (clusterValue[i] - clusterValue[j]);
            diff += sqrt((R[0]*R[0] + R[1]*R[1] + R[2]*R[2]))*histogram[j];
        }
        colorDist[i] = diff;
        if (diff<min) min = diff;
        if (diff>max) max = diff;

        //CENTERS OF COLORS
        Point center = Point(colorCentersMinMax[i][0].x  + (colorCentersMinMax[i][1].x - colorCentersMinMax[i][0].x)/2,colorCentersMinMax[i][0].y + (colorCentersMinMax[i][1].y - colorCentersMinMax[i][0].y)/2);
        colorCenters[i] = center;
    }

    //NORMALIZE COLORS' DISTANCE FROM 0 TO 255

    for (int i=0; i<clusterCount; i++){
        colorDist[i] = ((colorDist[i] - min)*(255/(max-min)));
    }

    for( int y = 0; y < im.rows; y++ )
        for( int x = 0; x < im.cols; x++ )
        {
            int cluster_idx = labels.at<int>(y + x*im.rows,0);
            colorDistance.at<Vec3b>(y,x)[0] = colorDist[cluster_idx];
            colorDistance.at<Vec3b>(y,x)[1] = colorDist[cluster_idx];
            colorDistance.at<Vec3b>(y,x)[2] = colorDist[cluster_idx];

            //----------------------

            Point center = Point(im.cols/2,im.rows/2);
            Point colorCenter = colorCenters[cluster_idx];

            double v_x = colorCentersMinMax[cluster_idx][1].x - colorCentersMinMax[cluster_idx][0].x;
            double v_y = colorCentersMinMax[cluster_idx][1].y - colorCentersMinMax[cluster_idx][0].y;
            double d_x = abs(center.x - colorCenter.x);
            double d_y = abs(center.y - colorCenter.y);
            double distanceFromCenter = sqrt(d_x*d_x + d_y*d_y);



            //distanceFromCenter = 1/distanceFromCenter;
            distanceFromCenter /= 255;
            distanceFromCenter = 1 - distanceFromCenter;
            //distanceFromCenter = exp(- distanceFromCenter / 2);
            distanceFromCenter = 255*(distanceFromCenter-1)+255;

            //cout << distanceFromCenter << endl;

            distFromCenter.at<Vec3b>(y,x)[0] = distanceFromCenter;
            distFromCenter.at<Vec3b>(y,x)[1] = distanceFromCenter;
            distFromCenter.at<Vec3b>(y,x)[2] = distanceFromCenter;

        }
    cvtColor(im,im,cv::COLOR_Lab2BGR);
    //imshow("global contrast", colorDistance);
    //waitKey(0);
    //imwrite("./img/dist.jpg", distFromCenter);
    //waitKey(0);

    imwrite("./img/kwantyzacja_K.png", im);
    return colorDistance;
}

// HISTOGRAM //
void calculateHistogram(Mat im,
                        Mat &averageX,
                        Mat &averageY,
                        Mat &averageX2,
                        Mat &averageY2,
                        vector<float> &LL,
                        vector<float> &AA,
                        vector<float> &BB,
                        Mat &histogram,
                        Mat &histogramIndex){

    Mat lab, Lshift, Ashift, Bshift;

    double minL, maxL, minA, maxA, minB, maxB;

    averageX        = Mat::zeros(1, histogramSize3D, CV_32FC1);
    averageY        = Mat::zeros(1, histogramSize3D, CV_32FC1);
    averageX2       = Mat::zeros(1, histogramSize3D, CV_32FC1);
    averageY2       = Mat::zeros(1, histogramSize3D, CV_32FC1);

    cvtColor(im, lab, CV_BGR2Lab);
    split(lab, LAB);

    minMaxLoc(LAB[0], &minL, &maxL);
    minMaxLoc(LAB[1], &minA, &maxA);
    minMaxLoc(LAB[2], &minB, &maxB);

    float tempL = (255 - maxL + minL) / (maxL - minL + 1e-3);
    float tempA = (255 - maxA + minA) / (maxA - minA + 1e-3);
    float tempB = (255 - maxB + minB) / (maxB - minB + 1e-3);

    Lshift          = Mat::zeros(1, 256, CV_32SC1);
    Ashift          = Mat::zeros(1, 256, CV_32SC1);
    Bshift          = Mat::zeros(1, 256, CV_32SC1);

    for (int i = 0; i < 256; i++) {
        Lshift.at<int>(0,i) = tempL * (i - minL) - minL;
        Ashift.at<int>(0,i) = tempA * (i - minA) - minA;
        Bshift.at<int>(0,i) = tempB * (i - minB) - minB;

    }

    // LAB //

    minL = minL / 2.56;
    maxL = maxL / 2.56;
    minA = minA - 128;
    maxA = maxA - 128;
    minB = minB - 128;
    maxB = maxB - 128;

    tempL   = float(maxL - minL)/histogramSize1D;
    tempA   = float(maxA - minA)/histogramSize1D;
    tempB   = float(maxB - minB)/histogramSize1D;

    float sL = float(maxL - minL)/histogramSize1D/2 + minL;
    float sA = float(maxA - minA)/histogramSize1D/2 + minA;
    float sB = float(maxB - minB)/histogramSize1D/2 + minB;

    for (int i = 0; i < histogramSize3D; i++) {

        int lpos = i % histogramSize1D;
        int apos = i % histogramSize2D / histogramSize1D;
        int bpos = i / histogramSize2D;

        LL.push_back(lpos * tempL + sL);
        AA.push_back(apos * tempA + sA);
        BB.push_back(bpos * tempB + sB);

    }

    // LAB HISTOGRAM //

    histogramIndex          = Mat::zeros(im.rows, im.cols, CV_32SC1);
    histogram               = Mat::zeros(1, histogramSize3D, CV_32SC1);

    int*    histogramPtr    = histogram.ptr<int>(0);

    float* averageXPtr      = averageX.ptr<float>(0);
    float* averageYPtr      = averageY.ptr<float>(0);
    float* averageX2Ptr     = averageX2.ptr<float>(0);
    float* averageY2Ptr     = averageY2.ptr<float>(0);

    int*    LshiftPtr       = Lshift.ptr<int>(0);
    int*    AshiftPtr       = Ashift.ptr<int>(0);
    int*    BshiftPtr       = Bshift.ptr<int>(0);

    int histShift = 8 - logSize;

    for (int y = 0; y < im.rows; y++) {
        int*    histogramIndexPtr   = histogramIndex.ptr<int>(y);
        uchar*    LPtr   = LAB[0].ptr<uchar>(y);
        uchar*    APtr   = LAB[1].ptr<uchar>(y);
        uchar*    BPtr   = LAB[2].ptr<uchar>(y);

        for (int x = 0; x < im.cols; x++) {
            int lpos                = (LPtr[x] + LshiftPtr[LPtr[x]]) >> histShift;
            int apos                = (APtr[x] + AshiftPtr[APtr[x]]) >> histShift;
            int bpos                = (BPtr[x] + BshiftPtr[BPtr[x]]) >> histShift;
            int index               = lpos + (apos << logSize) + (bpos << logSize2);
            histogramIndexPtr[x]    = index;
            histogramPtr[index]++;
            averageXPtr[index]      += x;
            averageYPtr[index]      += y;
            averageX2Ptr[index]     += squaresPtr[x];
            averageY2Ptr[index]     += squaresPtr[y];

        }
    }
}

int precomputeParameters(Mat histogram,
                         vector<float> LL,
                         vector<float> AA,
                         vector<float> BB,
                         int numberOfPixels,
                         vector<int> &reverseMap,
                         Mat &map,
                         Mat &colorDistance,
                         Mat &exponentialColorDistance){



    int*    histogramPtr    = histogram.ptr<int>(0);
    Mat problematic         = Mat::zeros(histogram.cols, 1, CV_32SC1);
    Mat closestElement      = Mat::zeros(histogram.cols, 1, CV_32SC1);
    Mat sortedHistogramIdx;

    // COLOR NUMBER REDUCTION //

    sortIdx(histogram, sortedHistogramIdx, CV_SORT_EVERY_ROW + CV_SORT_DESCENDING);
    int* sortedHistogramIdxPtr = sortedHistogramIdx.ptr<int>(0);
    float energy = 0;
    int binCountThreshold = 0;
    float energyThreshold = 0.95 * numberOfPixels;

    for (int i = 0; i < histogram.cols; i++) {
        energy += (float)histogramPtr[sortedHistogramIdxPtr[i]];

        if (energy > energyThreshold){
            binCountThreshold = histogramPtr[sortedHistogramIdx.at<int>(0,i)];
            break;
        }
    }

    for (int i = 0; i < histogram.cols; i++)
        if (histogramPtr[i] < binCountThreshold)
            problematic.at<int>(i,0) = 1;

    map = Mat::zeros(1, histogram.cols, CV_32SC1);
    int* mapPtr = map.ptr<int>(0);
    int count = 0;

    for (int i = 0; i < histogram.cols; i++) {
        if (histogramPtr[i] >= binCountThreshold) {
            L.push_back(LL[i]);
            A.push_back(AA[i]);
            B.push_back(BB[i]);

            mapPtr[i] = count;
            reverseMap.push_back(i);
            count++;
        }
        else if(histogramPtr[i] < binCountThreshold && histogramPtr[i] > 0){
            float mini = 1e6;
            int closest = 0;
            for (int k = 0; k < histogram.cols; k++) {
                if (!problematic.at<int>(k,0)){
                    float dd = pow((LL[i] - LL[k]),2) + pow((AA[i] - AA[k]),2) + pow((BB[i] - BB[k]),2);

                    if (dd < mini) {
                        mini = dd;
                        closest = k;
                    }
                }
            }
            closestElement.at<int>(i,0) = closest;
        }

    }



    for (int i = 0; i < histogram.cols; i++)
        if(problematic.at<int>(i,0))
            mapPtr[i] = mapPtr[closestElement.at<int>(i,0)];

    int numberOfColors = (int)L.size();

    // COLOR WEIGHTS //
    exponentialColorDistance    = Mat::zeros(numberOfColors, numberOfColors, CV_32FC1);
    colorDistance               = Mat::zeros(numberOfColors, numberOfColors, CV_32FC1);

    for (int i = 0; i < numberOfColors; i++) {
        colorDistance.at<float>(i,i)            = 0;
        exponentialColorDistance.at<float>(i,i) = 1.0;

        for (int k = i + 1; k < numberOfColors; k++) {
            float colorDifference                   = pow(L[i] - L[k],2) + pow(A[i] - A[k],2) + pow(B[i] - B[k],2);
            colorDistance.at<float>(i,k)            = sqrt(colorDifference);
            colorDistance.at<float>(k,i)            = sqrt(colorDifference);
            exponentialColorDistance.at<float>(i,k) = exp(- colorDifference / (2 * sigmac * sigmac));
            exponentialColorDistance.at<float>(k,i) = exponentialColorDistance.at<float>(i,k);
        }
    }


    return numberOfColors;
}

// BILATERAL FILTERING //

void bilateralFiltering(Mat colorDistance,
                        Mat exponentialColorDistance,
                        vector<int> reverseMap,
                        int* histogramPtr,
                        float* averageXPtr,
                        float* averageYPtr,
                        float* averageX2Ptr,
                        float* averageY2Ptr,
                        Mat &mx,
                        Mat &my,
                        Mat &Vx,
                        Mat &Vy,
                        Mat &contrast){

    int numberOfColors = colorDistance.cols;

    Mat X       = Mat::zeros(1, numberOfColors, CV_32FC1);
    Mat Y       = Mat::zeros(1, numberOfColors, CV_32FC1);
    Mat X2      = Mat::zeros(1, numberOfColors, CV_32FC1);
    Mat Y2      = Mat::zeros(1, numberOfColors, CV_32FC1);
    Mat NF      = Mat::zeros(1, numberOfColors, CV_32FC1);

    float* XPtr     = X.ptr<float>(0);
    float* YPtr     = Y.ptr<float>(0);
    float* X2Ptr    = X2.ptr<float>(0);
    float* Y2Ptr    = Y2.ptr<float>(0);
    float* NFPtr    = NF.ptr<float>(0);

    // COLOR CONTRAST, SPATIAL CENTER, VARIANCE //

    contrast    = Mat::zeros(1, numberOfColors, CV_32FC1);
    float* contrastPtr  = contrast.ptr<float>(0);

    for (int i = 0; i < numberOfColors; i++) {
        float* colorDistancePtr             = colorDistance.ptr<float>(i);
        float* exponentialColorDistancePtr  = exponentialColorDistance.ptr<float>(i);

        for (int k = 0; k < numberOfColors; k++) {
            contrastPtr[i]  += colorDistancePtr[k] * histogramPtr[reverseMap[k]];
            XPtr[i]         += exponentialColorDistancePtr[k] * averageXPtr[reverseMap[k]];
            YPtr[i]         += exponentialColorDistancePtr[k] * averageYPtr[reverseMap[k]];
            X2Ptr[i]        += exponentialColorDistancePtr[k] * averageX2Ptr[reverseMap[k]];
            Y2Ptr[i]        += exponentialColorDistancePtr[k] * averageY2Ptr[reverseMap[k]];
            NFPtr[i]        += exponentialColorDistancePtr[k] * histogramPtr[reverseMap[k]];
        }
    }

    divide(X,   NF, X);
    divide(Y,   NF, Y);
    divide(X2,  NF, X2);
    divide(Y2,  NF, Y2);

    X.assignTo(mx);
    Y.assignTo(my);

    Vx = X2 - mx.mul(mx);
    Vy = Y2 - my.mul(my);
}

// PROBABILITY OF SALIENCY //

void calculateProbability(Mat mx,
                          Mat my,
                          Mat Vx,
                          Mat Vy,
                          Mat modelMean,
                          Mat modelInverseCovariance,
                          int width,
                          int height,
                          Mat &Xsize,
                          Mat &Ysize,
                          Mat &Xcenter,
                          Mat &Ycenter,
                          Mat &shapeProbability){

    sqrt(12 * Vx, Xsize);
    Xsize       = Xsize/(float)width;

    sqrt(12 * Vy, Ysize);
    Ysize       = Ysize/(float)height;

    Xcenter = (mx - width /2)/(float)width;
    Ycenter = (my - height/2)/(float)height;

    Mat     g;

    vconcat(Xsize, Ysize, g);
    vconcat(g, Xcenter, g);
    vconcat(g, Ycenter, g);

    Mat repeatedMeanVector;

    repeat(modelMean, 1, Xcenter.cols, repeatedMeanVector);
    g = g - repeatedMeanVector;
    g = g/2;

    shapeProbability = Mat::zeros(1, Xcenter.cols, CV_32FC1);
    float* shapeProbabilityPtr = shapeProbability.ptr<float>(0);

    for (int i = 0; i < Xcenter.cols; i++) {
        Mat result, transposed;
        transpose(g.col(i), transposed);
        gemm(transposed, modelInverseCovariance, 1.0, 0.0, 0.0, result);
        gemm(result, g.col(i), 1.0, 0.0, 0.0, result);
        shapeProbabilityPtr[i] = exp(- result.at<float>(0,0) / 2);
    }
}

// SALIENCY MAP //


Mat computeGlobalContrast(Mat contrast, Mat exponentialColorDistance, Mat histogramIndex, int* mapPtr, Mat image){

    double minVal, maxVal;
    int numberOfColors  = contrast.cols;
    float* saliencyPtr  = contrast.ptr<float>(0);

    for (int i = 0; i < numberOfColors; i++) {
        float a1, a2;
        a1 = 0; a2 =0;
        for (int k = 0; k < numberOfColors; k++) {
            if (exponentialColorDistance.at<float>(i,k) > 0.0){
                a1 += saliencyPtr[k] * exponentialColorDistance.at<float>(i,k);
                a2 += exponentialColorDistance.at<float>(i,k);
            }
        }
        saliencyPtr[i] = a1/a2;
    }

    minMaxLoc(contrast, &minVal, &maxVal);
    contrast = contrast - minVal;

    if (minVal == 0 && maxVal == 0) contrast = 0;
    else contrast = 255 * contrast / (maxVal - minVal) + 1e-3;
    minMaxLoc(contrast, &minVal, &maxVal);

    Mat contrastMat = Mat::zeros(image.rows, image.cols, CV_8UC1);

    for (int y = 0; y < image.rows; y++){
        uchar* SMPtr = contrastMat.ptr<uchar>(y);
        int* histogramIndexPtr = histogramIndex.ptr<int>(y);
        for (int x = 0; x < image.cols; x++){
            float sal = saliencyPtr[mapPtr[histogramIndexPtr[x]]];
            SMPtr[x] = (uchar)(sal);
        }
    }
    equalizeHist(contrastMat,contrastMat);
    GaussianBlur(contrastMat,contrastMat,Size(51,51),0);

    //imshow("contrastMat", contrastMat);
    //waitKey(0);
    return contrastMat;
}

Mat computeDistanceFromCenter(Mat shapeProbability, Mat exponentialColorDistance, Mat histogramIndex, int* mapPtr, Mat image){
    double minVal, maxVal;
    int numberOfColors  = shapeProbability.cols;

    float* saliencyPtr  = shapeProbability.ptr<float>(0);

    for (int i = 0; i < numberOfColors; i++) {
        float a1, a2;
        a1 = 0; a2 =0;
        for (int k = 0; k < numberOfColors; k++) {
            if (exponentialColorDistance.at<float>(i,k) > 0.0){
                a1 += saliencyPtr[k] * exponentialColorDistance.at<float>(i,k);
                a2 += exponentialColorDistance.at<float>(i,k);
            }
        }
        saliencyPtr[i] = a1/a2;
    }

    minMaxLoc(shapeProbability, &minVal, &maxVal);
    shapeProbability = shapeProbability - minVal;

    if (minVal == 0 && maxVal == 0) shapeProbability= 0;
    else shapeProbability = 255 * shapeProbability / (maxVal - minVal) + 1e-3;
    minMaxLoc(shapeProbability, &minVal, &maxVal);

    Mat gaussianMat = Mat::zeros(image.rows, image.cols, CV_8UC1);

    for (int y = 0; y < image.rows; y++){
        uchar* SMPtr = gaussianMat.ptr<uchar>(y);
        int* histogramIndexPtr = histogramIndex.ptr<int>(y);
        for (int x = 0; x < image.cols; x++){
            float sal = saliencyPtr[mapPtr[histogramIndexPtr[x]]];
            SMPtr[x] = (uchar)(sal);
        }
    }

    equalizeHist(gaussianMat,gaussianMat);
    GaussianBlur(gaussianMat,gaussianMat,Size(51,51),0);
    //imshow("gaussianMat", gaussianMat);
    //waitKey(0);

    // vignette

    Mat gB(gaussianMat.rows, gaussianMat.cols, CV_8UC1, Scalar(0,0,0));

    RotatedRect rotRec(Point2f(gaussianMat.cols/2, gaussianMat.rows/2),Size2f(gaussianMat.cols/1.1, gaussianMat.rows/1.1), 0.0f);
    ellipse(gB, rotRec, Scalar(32,32,32), -1);
    RotatedRect rotRec2(Point2f(gaussianMat.cols/2, gaussianMat.rows/2),Size2f(gaussianMat.cols/1.2, gaussianMat.rows/1.5), 0.0f);
    ellipse(gB, rotRec2, Scalar(64,64,64), -1);
    RotatedRect rotRec3(Point2f(gaussianMat.cols/2, gaussianMat.rows/2),Size2f(gaussianMat.cols/1.5, gaussianMat.rows/4), 0.0f);
    ellipse(gB, rotRec3, Scalar(82,82,82), -1);

    GaussianBlur(gB,gB,Size(1081,1081),0);
    gaussianMat = gaussianMat.mul(gB,0.01);


    return gaussianMat;
}

static void read_csv(const string& filename, vector<Mat>& images, vector<int>& labels, char separator = ',') { // trzeba uwżać jaki jest w pliku
    std::ifstream file(filename.c_str(), ifstream::in);
    if (!file) {
        string error_message = "No valid input file was given, please check the given filename.";
        CV_Error(CV_StsBadArg, error_message);
    }
    string line, path, classlabel;
    while (getline(file, line)) {
        stringstream liness(line);
        getline(liness, path, separator);
        getline(liness, classlabel);
        if(!path.empty() && !classlabel.empty()) {
            images.push_back(imread(path, 0));
            labels.push_back(atoi(classlabel.c_str()));
        }
    }
}

Mat detectFaces(Mat image){
    string fn_haar = "./haarcascade_frontalface_default.xml";
    string fn_haar_profile = "./haarcascade_profileface.xml";
    string eye_cascade = "./haarcascade_eye.xml";
    string eyeglasses_cascade = "./haarcascade_eye_tree_eyeglasses.xml";
    string nose_cascade = "./haarcascade_mcs_nose.xml";
    string mouth_cascade = "./haarcascade_mcs_mouth.xml";

    vector< Rect_<int> > faces;
    vector< Rect_<int> > faces_temp;
    vector< Rect_<int> > profile_faces;
    vector< Rect_<int> > eyes;
    vector< Rect_<int> > eyeglasses;
    vector< Rect_<int> > mouths;
    vector< Rect_<int> > noses;

    CascadeClassifier haar_cascade;
    CascadeClassifier haar_cascade_profile;
    CascadeClassifier haar_cascade_eye;
    haar_cascade.load(fn_haar);
    haar_cascade_profile.load(fn_haar_profile);
    haar_cascade_eye.load(eye_cascade);

    Mat gray, flipedGray, rotLeftGray, rotRightGray;
    cvtColor(image, gray, COLOR_BGR2GRAY);

    //string imgName = "./img/gray.jpg";
    //imwrite(imgName, gray);

    equalizeHist(gray,gray);

    flip(gray,flipedGray,1);
    Mat rotM = getRotationMatrix2D(Point(gray.cols/2,gray.rows/2),20,1);
    warpAffine(gray,rotRightGray,rotM,Point(gray.cols,gray.rows));
    rotM = getRotationMatrix2D(Point(gray.cols/2,gray.rows/2),-20,1);
    warpAffine(gray,rotLeftGray,rotM,Point(gray.cols,gray.rows));

    /*
    imgName = "./img/gray_equalized.jpg";
    imwrite(imgName, gray);

    imgName = "./img/gray_flipped.jpg";
    imwrite(imgName, flipedGray);

    imgName = "./img/gray_rotated_left.jpg";
    imwrite(imgName, rotLeftGray);

    imgName = "./img/gray_rotated_right.jpg";
    imwrite(imgName, rotRightGray);
*/

    // FRONTFACE
    haar_cascade.detectMultiScale(gray, faces, 1.1, 21,0,Size(10,10));

    haar_cascade.detectMultiScale(flipedGray, faces_temp, 1.1, 21,0,Size(10,10));
    faces.insert(faces.end(),faces_temp.begin(),faces_temp.end());

    haar_cascade.detectMultiScale(rotLeftGray, faces_temp, 1.1, 21,0,Size(10,10));
    faces.insert(faces.end(),faces_temp.begin(),faces_temp.end());

    haar_cascade.detectMultiScale(rotRightGray, faces_temp, 1.1, 21,0,Size(10,10));
    faces.insert(faces.end(),faces_temp.begin(),faces_temp.end());

    // PROFILE
    haar_cascade_profile.detectMultiScale(gray, profile_faces, 1.1, 21,0,Size(10,10));

    Mat temp = Mat::zeros(image.rows, image.cols, CV_8UC1);
    //Find face profiles
    for(int i = 0; i < profile_faces.size(); i++) {
        Rect face_i = profile_faces[i];
        RotatedRect rotRec(Point2f(face_i.x + face_i.width/2, face_i.y + face_i.height/2),Size2f(face_i.width*1.1, face_i.height*1.5), 0.0f);
        ellipse(temp, rotRec, Scalar(63,63,63), -1);
    }

    // Find the faces in the frame:
    for(int i = 0; i < faces.size(); i++) {
        Rect face_i = faces[i];
        Mat face = gray(face_i);

        // draw ellipse around face
        RotatedRect rotRec(Point2f(face_i.x + face_i.width/2, face_i.y + face_i.height/2),Size2f(face_i.width*1.1, face_i.height*1.5), 0.0f);
        ellipse(temp, rotRec, Scalar(63,63,63), -1);

        // detect eyes, mouths and noses inside face
        haar_cascade_eye.detectMultiScale(face,eyes);
        haar_cascade_eye.load(eyeglasses_cascade);
        haar_cascade_eye.detectMultiScale(face,eyeglasses);
        haar_cascade_eye.load(mouth_cascade);
        haar_cascade_eye.detectMultiScale(face,mouths);
        haar_cascade_eye.load(nose_cascade);
        haar_cascade_eye.detectMultiScale(face,noses);

        //detect eyes
        for(int j = 0; j < eyes.size(); j++){
            Rect eye_j = eyes[j];
            RotatedRect rotRec2(Point2f(face_i.x + eye_j.x + eye_j.width/2, face_i.y + eye_j.y + eye_j.height/2),Size2f(eye_j.width, eye_j.height*0.5), 0.0f);
            ellipse(temp, rotRec2, Scalar(255,255,255), -1);
        }

        //detect eyeglasses
        for(int j = 0; j < eyeglasses.size(); j++){
            Rect eye_j = eyeglasses[j];
            RotatedRect rotRec2(Point2f(face_i.x + eye_j.x + eye_j.width/2, face_i.y + eye_j.y + eye_j.height/2),Size2f(eye_j.width, eye_j.height*0.5), 0.0f);
            ellipse(temp, rotRec2, Scalar(255,255,255), -1);
        }

        //detect mouths
        for(int j = 0; j < mouths.size(); j++){
            Rect eye_j = mouths[j];
            RotatedRect rotRec2(Point2f(face_i.x + eye_j.x + eye_j.width/2, face_i.y + eye_j.y + eye_j.height/2),Size2f(eye_j.width, eye_j.height*0.5), 0.0f);
            ellipse(temp, rotRec2, Scalar(255,255,255), -1);
        }

        //detect noses
        for(int j = 0; j < noses.size(); j++){
            Rect eye_j = noses[j];
            RotatedRect rotRec2(Point2f(face_i.x + eye_j.x + eye_j.width/2, face_i.y + eye_j.y + eye_j.height/2),Size2f(eye_j.width, eye_j.height*0.5), 0.0f);
            ellipse(temp, rotRec2, Scalar(255,255,255), -1);
        }

    }
    GaussianBlur(temp,temp,Size(71,71),0);
    return temp;
}

Mat detectMotion(Mat firstFrame, Mat secondFrame){
    Mat temp = Mat::zeros(firstFrame.rows, firstFrame.cols, CV_8UC1);
    Mat flow;
    int step = 16;
    calcOpticalFlowFarneback(firstFrame,secondFrame, flow, 0.5, 3, 15, 3, 5, 1.2, 0);

    for(int y = 0; y < firstFrame.rows; y += step){
        for(int x = 0; x < firstFrame.cols; x += step)
        {
            const Point2f& fxy = flow.at<Point2f>(y, x);
            ellipse(temp,Point(x,y),Size(abs(fxy.x)*2,abs(fxy.y)*2),0,0,360,Scalar(255,255,255), -1);
        }
    }
    GaussianBlur(temp,temp,Size(51,51),0);
    return temp;
}

void setLabel(Mat& im, const string label, const Point& p){
    float marginLeft = im.cols*20/1000;
    float fontScale = (im.cols - (im.cols - marginLeft))*0.03;

    int fontface = cv::FONT_HERSHEY_SIMPLEX;
    double scale = 1.5*fontScale;
    int thickness = 4;
    int baseline = 0;

    Size text = getTextSize(label, fontface, scale, thickness, &baseline);
    rectangle(im,p + Point(0, baseline), p + Point(text.width, -text.height), CV_RGB(0,0,0), CV_FILLED);
    putText(im, label, p, fontface, scale, CV_RGB(255,255,255), thickness, 8);
}

// PROGRAM DESCRIPTION //

void outputHowToUse(){
    cout << "How to Use:" << endl;
    cout << "./Test -c /path/to/csv/folder/ -p /path/to/video/file -o 0|1|2|3" << endl;
    cout << "Parameter -c is optional. If there is no -c parameter then the default folder is ./csv/" << endl;
    cout << "Parameter -o is optional. If there is no -o parameter then as output will be used 'divided in four' version of video." << endl;
    cout << "0 = video divided in four; 1 = saliency map only; 2 = saliency map blended with video; 3 = saliency map blended with video and occulography." << endl;
}

void split(const string& s, char c, vector<string>& v) {
    string::size_type i = 0;
    string::size_type j = s.find(c);

    while (j != string::npos) {
        v.push_back(s.substr(i, j-i));
        i = ++j;
        j = s.find(c, j);

        if (j == string::npos)
            v.push_back(s.substr(i, s.length()));
    }
}

// MAIN FUNCTION //

int main(int argc, const char * argv[]){
    cout << "--------------------------------------" << endl;
    cout << "SALIENCY MAP | v 02/2018" << endl;
    cout << "--------------------------------------" << endl;
    if(argc < 2){
        outputHowToUse();
        return 0;
    }
    else{
        if(argc > 9){
            outputHowToUse();
            return 0;
        }
        else{
            for(int i = 1; i < argc; i += 2){
                const char* input = argv[i];

                if(!strcmp(input,"-p")){
                    dataPath = argv[i + 1];
                }
                else if(!strcmp(input,"-c")){
                    csvPath = argv[i + 1];
                }
                else if(!strcmp(input,"-o")){
                    outputType = argv[i + 1];
                }
                else if(!strcmp(input,"-w")){
                    string weight = argv[i + 1];
                    vector<string> v;
                    if(weight.size()>0) {
                        split(weight, ',', v);
                        w_globalContrast = stof(v[0]);
                        if (v.size()>1) w_gaussianDist = stof(v[1]);
                        if (v.size()>2) w_face = stof(v[2]);
                        if (v.size()>3) w_movement = stof(v[3]);

                       // weights auto correction [0,1], w1+w2+w3+w4<=1
                        if(w_globalContrast<0) w_globalContrast = 0;
                        if(w_gaussianDist<0) w_gaussianDist = 0;
                        if(w_face<0) w_face = 0;
                        if(w_movement<0) w_movement = 0;

                        if(w_globalContrast>=1){
                            w_globalContrast = 1;
                            w_gaussianDist, w_face, w_movement = 0;
                        }
                        if(w_globalContrast + w_gaussianDist > 1){
                            w_gaussianDist = 1 - w_globalContrast;
                            w_face, w_movement = 0;
                        }
                        if(w_globalContrast + w_gaussianDist + w_face > 1){
                            w_face = 1 - (w_globalContrast + w_gaussianDist);
                            w_movement = 0;
                        }
                        if(w_globalContrast + w_gaussianDist + w_face + w_movement > 1){
                            w_movement = 1 - (w_globalContrast + w_gaussianDist + w_face);
                        }
                    }
                }
                else{
                    cout << "Invalid input type " << input << endl;
                    outputHowToUse();
                    return 0;
                }
            }
        }
    }

    cout << "--------------------------------------" << endl;
    cout << "Data path: " << dataPath << endl;
    cout << "Occulography csv path: " << csvPath << endl;
    cout << "Weights | Global contrast: " << w_globalContrast << " Gaussian distribussion: " << w_gaussianDist << " Face: " << w_face << " Movement: " << w_movement << endl;
    cout << "--------------------------------------" << endl;

    clock_t st, et;

    vector<string> imagePaths;
    vector<string> imageNames;
    VideoCapture cap;

    readFiles(imagePaths, imageNames, cap);
    cout << "Processing..." << endl;

    float totalColor    = 0;
    float totalPixels   = 0;
    float totalTime     = 0;

    for (int i = 0; i < squares.cols; i++)
        squaresPtr[i] = pow(i,2);

    // POINT OF INTEREST //
    struct POI{
        double ms;
        double x,y;
        int color;
    };

    // CSV LOADING // add parameter check
    string input;
    vector<POI> ellipseList;
    vector<string> csvPaths;
    ofstream metricsCSV;

    if (csvPath != "no_csv_data"){
        DIR *csvDir = opendir(csvPath.c_str());

        for( dirent *imp = readdir(csvDir); imp != NULL; imp = readdir(csvDir) ) {
            string csvName = imp->d_name;
            bool check = csvName.size() > 3 && csvName.substr( csvName.size()-3 ) == "csv";
            if(csvName.size() > 3 && check){
                string temp = csvPath + csvName;
                csvPaths.push_back(temp);
            }
        }

        for (int i=0; i<csvPaths.size(); i++){
            ifstream csvFile(csvPaths[i]);
            if (csvFile.is_open()){
                while (!csvFile.eof()) {
                    csvFile >> input;
                    char delim = ';';
                    stringstream ss;
                    ss.str(input);
                    int j = 3;
                    POI e;
                    while (getline(ss,input,delim) && j>0) {
                        if (j==3){e.ms = atof(input.c_str());}
                        else if (j==2) {e.x = atof(input.c_str());}
                        else {
                            e.y = atof(input.c_str());
                            e.color = i*10;
                            ellipseList.push_back(e);
                        }
                        j--;
                    }
                }
            }
            else cout << "Csv file cannot be opened!" << endl;
        }
        // create new output csv with metrics for difference between saliency and ground truth map
        //metricsCSV.open ("metrics.csv");
        //metricsCSV << "MSE,CCORR,LCC,KLD,J\n";
        //metricsCSV << "LCC_global,LCC_contrast,LCC_distance_from_middle,LCC_face,LCC_movement\n";
    }

    // FACE DETECTION PREPARATION
    /*
    string fn_csv = "./faceCSV.csv";

    vector<Mat> images;
    vector<int> labels;
    try {
        read_csv(fn_csv, images, labels);
        cout << "faceCSV loaded" << endl;
    } catch (cv::Exception& e) {
        cerr << "Error opening file \"" << fn_csv << "\". Reason: " << e.msg << endl;
        exit(1);
    }
    Ptr<FaceRecognizer> model = createFisherFaceRecognizer();
    model->train(images, labels);
    CascadeClassifier haar_cascade;
    */
    // END OF FACE DETECTION PREPARATION

    Mat contrastMat, gaussianMat, facesMat, movementMat;

    Mat prevImage;
    int imagesCounter = 0;
    // do testow tylko 1/4 filmu
    //for (int totalImages = 0; totalImages < (imagePaths.size()); ++totalImages) {


for (int totalImages = 0; totalImages < imagePaths.size(); ++totalImages) {

        st = clock();
        Mat im;
        cap >> im;

       // Mat im = imread("./img/kwantyzacja_2.png");

        if(im.data){
            totalPixels += im.cols*im.rows;

            // SALIENCY MAP COMPUTATION  //

            LAB.clear();
            L.clear();
            A.clear();
            B.clear();

            Mat averageX, averageY, averageX2, averageY2, histogram, histogramIndex;
            vector<float> LL, AA, BB;

            calculateHistogram(im,averageX,averageY,averageX2,averageY2,LL,AA,BB,histogram,histogramIndex);

            float* averageXPtr      = averageX.ptr<float>(0);
            float* averageYPtr      = averageY.ptr<float>(0);
            float* averageX2Ptr     = averageX2.ptr<float>(0);
            float* averageY2Ptr     = averageY2.ptr<float>(0);

            int*    histogramPtr    = histogram.ptr<int>(0);
            Mat map, colorDistance, exponentialColorDistance;
            vector<int> reverseMap;

            int numberOfColors = precomputeParameters(histogram,LL,AA,BB,im.cols * im.rows,reverseMap,map,colorDistance,exponentialColorDistance);

            totalColor += numberOfColors;
            int* mapPtr = map.ptr<int>(0);
            Mat mx, my, Vx, Vy, contrast;

            bilateralFiltering(colorDistance,exponentialColorDistance,reverseMap,histogramPtr,averageXPtr,averageYPtr,averageX2Ptr,averageY2Ptr,mx,my,Vx,Vy,contrast);

            Mat Xsize, Ysize, Xcenter, Ycenter, shapeProbability;

            calculateProbability(mx,my,Vx,Vy,modelMean,modelInverseCovariance,im.cols,im.rows,Xsize,Ysize,Xcenter,Ycenter,shapeProbability);

            Mat SM = Mat(im.rows, im.cols, CV_8UC1);

            //GLOBAL CONTRAST
            contrastMat = computeGlobalContrast(contrast,exponentialColorDistance,histogramIndex,mapPtr, im);

            //contrastMat = globalContrast(im);
            //cvtColor(contrastMat,contrastMat, cv::COLOR_RGB2GRAY);


            //DISTANCE FROM CENTER
            if (w_gaussianDist != 0.0) gaussianMat = computeDistanceFromCenter(shapeProbability,exponentialColorDistance,histogramIndex,mapPtr, im);
            else gaussianMat = Mat(im.rows, im.cols, CV_8UC1, Scalar(0,0,0));
            //FACES
            if (w_face != 0.0) facesMat = detectFaces(im);
            else facesMat = Mat(im.rows, im.cols, CV_8UC1, Scalar(0,0,0));

            // source to BW
            Mat imBW;
            cvtColor(im, imBW, cv::COLOR_RGB2GRAY);
            //cvtColor(imBW, im, cv::COLOR_GRAY2RGB);

            //MOVEMENT
            if (w_movement != 0.0){
                if (totalImages>1) movementMat = detectMotion(prevImage,imBW);
                else movementMat = Mat(im.rows, im.cols, CV_8UC1, Scalar(0,0,0));
                imBW.copyTo(prevImage);
            }
            else movementMat = Mat(im.rows, im.cols, CV_8UC1, Scalar(0,0,0));

            //ADDING MAPS TO FINAL SALIENCY MAP
            SM = facesMat*w_face + movementMat*w_movement + contrastMat*w_globalContrast + gaussianMat*w_gaussianDist;
            equalizeHist(SM,SM);
            GaussianBlur(SM,SM,Size(51,51),0);

            //imshow("SM", SM);
            //waitKey(0);

            // END OF SALIENCY COMPUTATION //

            // DATA VISUALIZATION //

            // OUTPUT MATS
            Mat SMasHeatmap, occulographyOnHeatmap, occulography, bwSM, heatmapOnImage;
            Mat groundTruth = Mat::zeros(im.rows, im.cols, im.type());
            Mat groundTruthGreen = Mat::zeros(im.rows, im.cols, im.type());

            SM.copyTo(bwSM);            
            cvtColor(SM, SM, cv::COLOR_GRAY2RGB);
            applyColorMap(SM, SMasHeatmap, 2); //JET

            // blend heatmap and source
            double alpha = 0.5;
            double beta = 1.0 - alpha;

            addWeighted(im,alpha,SMasHeatmap,beta,0.0,heatmapOnImage);

            //??
            im.copyTo(occulography);
            SMasHeatmap.copyTo(occulographyOnHeatmap);

            // draw ellipse
            if (csvPath != "no_csv_data"){
                bool lift = true; // for damaged csv data lifts up fixation points
                for (int i=0; i<ellipseList.size()-1; i++){
                    POI e = ellipseList[i];
                    POI eNext = ellipseList[i+1];
                    int frameNumber = floor(((e.ms-300)*fps)/1000.0); // poprawka: - 0,3 s
                    int frameNumberMax = floor(((eNext.ms-300)*fps)/1000.0);

                    POI ePrev;
                    if (i>0) ePrev = ellipseList[i-1];
                    else ePrev.ms = 0; ePrev.x = 0; ePrev.y = 0; ePrev.color = 0;

                    frameNumber = ceil(ePrev.ms*fps/1000.0);
                    frameNumberMax = ceil(e.ms*fps/1000.0);

                    //cout << "Klatka poczatkowa: "<< frameNumber << " Klatka koncowa: "<< frameNumberMax << endl;
                    //cout << "Ms rekordu: " << e.ms << " Ms kolejnego rekordu: " << eNext.ms << endl;

                    if (totalImages >= frameNumber && totalImages <= frameNumberMax){
                        int radius = 50;
                        if(lift) e.y = e.y - 100000;
                        if(e.x == 0 || e.x == -1) radius = 0;
                        int x = e.x*im.cols/1000000;
                        int y = e.y*im.rows/1000000;
                        //int howLong = frameNumberMax - frameNumber;

                        Mat overlay;
                        Mat overlayGT;
                        double alpha = 0.3;
                        //occulographyOnHeatmap.copyTo(overlay);
                        groundTruth.copyTo(overlayGT);
                        groundTruthGreen.copyTo(overlay);


                        circle(overlay,Point(x, y), radius + 20, Scalar(255,50,255), -1);
                        circle(overlayGT,Point(x, y), radius + 20, Scalar(255,255,255), -1);
                        //circle(overlayGT,Point(x, y), 5, Scalar(255,255,255), -1);

                        //GaussianBlur(overlay,overlay,Size(281,281),0,0);

                        //addWeighted(overlay, alpha, occulographyOnHeatmap, 1 - alpha, 0, occulographyOnHeatmap);
                        addWeighted(overlay, alpha, groundTruthGreen, 1 - alpha, 0, groundTruthGreen);
                        addWeighted(overlayGT, alpha, groundTruth, 1 - alpha, 0, groundTruth);
                    }
                }
               GaussianBlur(groundTruth,groundTruth,Size(281,281),0,0);
               GaussianBlur(groundTruthGreen,groundTruthGreen,Size(281,281),0,0);
            }

            // METRICS CALCULTION //

            //normalize(bwSM, bwSM, 255, 0, NORM_MINMAX);
            //normalize(groundTruth, groundTruth, 255, 0, NORM_MINMAX);

            //imshow("groundTruth",groundTruth);
            //waitKey(0);

            /*

            Mat nGT(im.rows, im.cols, CV_8UC1);
            cout << nGT.depth() << endl;
            cout << nGT.channels() << endl << endl;

            groundTruth.convertTo(nGT, CV_8U);
            nGT.reshape(1);

            //imshow("groundTruth",nGT);
            //waitKey(0);
            for (int i=0; i<nGT.cols; i++){
                for (int j=0; j<nGT.rows; j++){
                    uchar value = nGT.at<uchar>(j,i);
                    if (value<=32) value = 0;
                    else value = 255;
                    nGT.at<uchar>(j,i) = value;
                }
            }
            */
            Mat nGT = groundTruth > 32;
            M.calculateAUC(nGT, bwSM,totalImages);

/*
            string imgName = "./AUC/SM_" + to_string(totalImages) + ".jpg";
            imwrite(imgName, SM);

            string imgName = "./AUC/rawGT_" + to_string(totalImages) + ".jpg";
            imwrite(imgName, groundTruth);

            imgName = "./AUC/nGT_" + to_string(totalImages) + ".jpg";
            imwrite(imgName, nGT);

*/
            cvtColor(bwSM, bwSM, cv::COLOR_GRAY2RGB);
/*
            imshow("groundTruth",nGT);
            waitKey(0);

            imshow("saliency",bwSM);
            waitKey(0);
*/
            //Mat nGT, nSM;

            //bwSM.convertTo(nSM, CV_32F);
            //double LCC_contrast, LCC_from_middle5, LCC_face, LCC_movement;



            //LCC = M.getLCC(nGT, nSM);
            //LCC = M.getLCC(groundTruth, bwSM);
            /*
            LCC_contrast = M.getLCC(nGT, contrastMat);
            LCC_from_middle = M.getLCC(nGT, gaussianMat);
            LCC_face = M.getLCC(nGT, facesMat);
            LCC_movement = M.getLCC(nGT, movementMat);
            */

            //cout << LCC << " " << endl;


            /*
            KLD = M.getKLD(nGT, nSM);
            J = M.getJ(nGT, nSM);
            MSE = M.getMSE(nGT, nSM);
            CCORR = M.getScalar(nGT, nSM);
            */
            //cout << LCC_movement << " ";

            //String text = to_string(MSE) + "," + to_string(CCORR) + "," + to_string(LCC) + "," + to_string(KLD) + "," + to_string(J) + "\n";
            //String text = to_string(LCC) + "," + to_string(LCC_contrast) + "," + to_string(LCC_from_middle) + "," + to_string(LCC_face) + "," + to_string(LCC_movement) + "\n";
            //metricsCSV << text;

            // OUTPUT TYPE //


            string label = "heatmap + eyetracking map";
            label = to_string(LCC);
            if (csvPath == "no_csv_data") {
                label = "no eyetracking data for presentation";
                Mat empty = Mat::zeros(im.rows, im.cols, im.type());
                empty.copyTo(occulographyOnHeatmap);
            }

            //imshow("x",occulographyOnHeatmap);
            //waitKey(0);
            applyColorMap(groundTruth, groundTruth, 11); //HOT

            // blend gT and source
            double a = 0.5;
            double b = 1.0 - a;
            addWeighted(im,a,groundTruth,b,0.0,groundTruth);

            a = 0.2;
            b = 1.0 - a;
            addWeighted(SMasHeatmap,a,groundTruthGreen,b,0.0,occulographyOnHeatmap);

            a = 0.3;
            b = 1.0 - a;
            //cvtColor(facesMat, facesMat, cv::COLOR_GRAY2RGB);
            addWeighted(imBW,a,facesMat,b,0.0,facesMat);

            float marginLeft = im.cols*20/1000;
            float marginBottom = im.rows - im.rows*20/1000;

            Point position = Point(marginLeft, marginBottom);

            setLabel(im,"original", position);

            stringstream stream;
            stream << fixed << setprecision(2) << w_globalContrast;
            string weight_w1 = stream.str();
            stream.str("");
            stream << fixed << setprecision(2) << w_gaussianDist;
            string weight_w2 = stream.str();
            stream.str("");
            stream << fixed << setprecision(2) << w_face;
            string weight_w3 = stream.str();
            stream.str("");
            stream << fixed << setprecision(2) << w_movement;
            string weight_w4 = stream.str();
            stream.str("");
            string weightsLabel = "saliency map " + weight_w1 + "-" + weight_w2 + "-" + weight_w3 + "-" + weight_w4;

            setLabel(SMasHeatmap,weightsLabel, position);
            setLabel(occulographyOnHeatmap,"eyetracking + saliency map", position);
            setLabel(heatmapOnImage,"combined (original + saliency map)", position);
            setLabel(groundTruth,"eyetracking map", position);
            setLabel(bwSM,"saliency map", position);

            Mat chosenVideo = Mat::zeros(im.rows, im.cols, im.type());

            if(outputType == "0"){ // wszystkie 4 mapy

                resize(groundTruth, groundTruth, Size(groundTruth.cols/2, groundTruth.rows/2));
                resize(im, im, Size(im.cols/2, im.rows/2));
                resize(heatmapOnImage, heatmapOnImage, Size(heatmapOnImage.cols/2, heatmapOnImage.rows/2));
                resize(occulographyOnHeatmap, occulographyOnHeatmap, Size(occulographyOnHeatmap.cols/2, occulographyOnHeatmap.rows/2));

                im.copyTo(chosenVideo(Rect(0,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                groundTruth.copyTo(chosenVideo(Rect(0,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
                heatmapOnImage.copyTo(chosenVideo(Rect(chosenVideo.cols/2,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                occulographyOnHeatmap.copyTo(chosenVideo(Rect(chosenVideo.cols/2,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
            }
            else if(outputType == "1"){ // sama heatmapa
                SMasHeatmap.copyTo(chosenVideo);
            }
            else if(outputType == "2"){ // heatmapa na tle filmu
                heatmapOnImage.copyTo(chosenVideo);
            }
            else if(outputType == "3"){ // czarno biała mapa atencji i czarno biała mapa ground truth
                //resize(SM, SM, Size(SM.cols/2, SM.rows/2));
                resize(im, im, Size(im.cols/2, im.rows/2));
                resize(bwSM, bwSM, Size(bwSM.cols/2, bwSM.rows/2));
                resize(groundTruth, groundTruth, Size(groundTruth.cols/2, groundTruth.rows/2));

                im.copyTo(chosenVideo(Rect(0,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                bwSM.copyTo(chosenVideo(Rect(0,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
                //SM.copyTo(chosenVideo(Rect(chosenVideo.cols/2,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                groundTruth.copyTo(chosenVideo(Rect(chosenVideo.cols/2,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
            }
            else if(outputType == "4"){ // 4 składowe mapy atencji

                setLabel(contrastMat,"contrast", position);
                setLabel(gaussianMat,"gaussian distribusion", position);
                setLabel(facesMat,"faces", position);
                setLabel(movementMat,"movement", position);

                resize(contrastMat, contrastMat, Size(contrastMat.cols/2, contrastMat.rows/2));
                resize(gaussianMat, gaussianMat, Size(gaussianMat.cols/2, gaussianMat.rows/2));
                resize(facesMat, facesMat, Size(facesMat.cols/2, facesMat.rows/2));
                resize(movementMat, movementMat, Size(movementMat.cols/2, movementMat.rows/2));

                cvtColor(contrastMat, contrastMat, cv::COLOR_GRAY2RGB);
                cvtColor(gaussianMat, gaussianMat, cv::COLOR_GRAY2RGB);
                cvtColor(movementMat, movementMat, cv::COLOR_GRAY2RGB);
                cvtColor(facesMat, facesMat, cv::COLOR_GRAY2RGB);

                contrastMat.copyTo(chosenVideo(Rect(0,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                gaussianMat.copyTo(chosenVideo(Rect(0,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
                facesMat.copyTo(chosenVideo(Rect(chosenVideo.cols/2,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                movementMat.copyTo(chosenVideo(Rect(chosenVideo.cols/2,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
            }
            else if(outputType == "5"){ // heatmapa, heatmapa na obrazie, czarno-biała mapa uwagi, mapa groundtruth
                resize(heatmapOnImage, heatmapOnImage, Size(heatmapOnImage.cols/2, heatmapOnImage.rows/2));
                resize(SMasHeatmap, SMasHeatmap, Size(SMasHeatmap.cols/2, SMasHeatmap.rows/2));
                resize(bwSM, bwSM, Size(bwSM.cols/2, bwSM.rows/2));
                resize(groundTruth, groundTruth, Size(groundTruth.cols/2, groundTruth.rows/2));

                SMasHeatmap.copyTo(chosenVideo(Rect(0,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                heatmapOnImage.copyTo(chosenVideo(Rect(chosenVideo.cols/2,0,chosenVideo.cols/2, chosenVideo.rows/2)));
                bwSM.copyTo(chosenVideo(Rect(0,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
                groundTruth.copyTo(chosenVideo(Rect(chosenVideo.cols/2,chosenVideo.rows/2,chosenVideo.cols/2, chosenVideo.rows/2)));
            }
            else if(outputType == "6a"){ //contrastMat
                setLabel(contrastMat,"contrast, intensity", position);
                cvtColor(contrastMat, contrastMat, cv::COLOR_GRAY2RGB);
                contrastMat.copyTo(chosenVideo);
            }
            else if(outputType == "6b"){ // gaussianMat
                setLabel(gaussianMat,"size and distribusion", position);
                cvtColor(gaussianMat, gaussianMat, cv::COLOR_GRAY2RGB);
                gaussianMat.copyTo(chosenVideo);
            }
            else if(outputType == "6c"){ // facesMat
                setLabel(facesMat,"faces", position);
                cvtColor(facesMat, facesMat, cv::COLOR_GRAY2RGB);
                facesMat.copyTo(chosenVideo);
            }
            else if(outputType == "6d"){ // movementMat
                setLabel(movementMat,"motion", position);
                cvtColor(movementMat, movementMat, cv::COLOR_GRAY2RGB);
                movementMat.copyTo(chosenVideo);
            }
            else if(outputType == "6e"){ // groudTruth
                setLabel(groundTruth,"eyetracking map", position);
                groundTruth.copyTo(chosenVideo);
            }

            outputVideo << chosenVideo;

            //imwrite("./img/output.png", chosenVideo);

            et = clock();
            totalTime += double(et-st)/CLOCKS_PER_SEC;
            imagesCounter = totalImages;
            if(totalImages % (imageNames.size()/10) == 0){
                float percentage = ((float) 100 * totalImages) / imageNames.size();
                cout << percentage << "%" << endl;
            }
        }
        else {
            cout << "Missed frame: " << totalImages;
            break;
        }
    }
    M.drawGlobalAUCplot();
    double globalAUC = M.getGlobalAUC(imagesCounter);
    M.saveAUC();

    //metricsCSV.close();
    cout << "Number of processed frames per one second: " << imagesCounter/ totalTime << endl;
    cout << "Total time: " << totalTime  << " [s] " << endl;
    return 0;
}

