#include "metric.h"
#include <fstream>

Metric::Metric() {
    for(int i=0; i<256; i++){
        this->whiteHistogram[i] = 0;
        this->blackHistogram[i] = 0;
    }
    this->globalAUCvalue = 0;
    this->whiteHistogramPixels = 0;
    this->blackHistogramPixels = 0;
}

double normalizeVal(double value, double min, double max){
    return (value - min)/(max - min);
}

double Metric::getLCC(const Mat& a, const Mat& b){ // Linear Corelation Coefficient
    Scalar meanA = mean(a);
    Scalar meanB = mean(b);
    Scalar intensityA = 0, intensityB = 0;
    double mA = meanA[0]/255.0;
    double mB = meanB[0]/255.0;
    double iA = 0, iB = 0, sumAB = 0, sumA2 = 0, sumB2 = 0;

    for (int i=0; i<a.cols; i++){
        for(int j=0; j<a.rows; j++){
            intensityA = a.at<uchar>(Point(i, j));
            intensityB = b.at<uchar>(Point(i, j));
            iA = intensityA[0]/255.0;
            iB = intensityB[0]/255.0;

            sumAB += (iA-mA)*(iB-mB);
            sumA2 += pow((iA-mA),2);
            sumB2 += pow((iB-mB),2);
        }
    }
    sumAB = sumAB/(a.cols*a.rows);
    sumA2 = sumA2/(a.cols*a.rows - 1);
    sumB2 = sumB2/(a.cols*a.rows - 1);
    if (sumA2 == 0 || sumB2 == 0 || sumAB == 0) return 0;
    else return sumAB/(sqrt(sumA2)*sqrt(sumB2));
}

void Metric::calculateAUC(const Mat& GT, const Mat& SM, int frameNumber){ // Area Under Curve
    unsigned long int whiteHistogramPixels = 0;
    unsigned long int  blackHistogramPixels = 0;
    unsigned long int  whiteHistogram[256];
    unsigned long int  blackHistogram[256];

    Mat AUCplot = Mat::zeros(500, 500, SM.type());

    for(int i=0; i<256; i++){
        whiteHistogram[i] = 0;
        blackHistogram[i] = 0;
    }

    for(int i=0; i<GT.cols; i++){
        for(int j=0; j<GT.rows; j++){

            Scalar index = GT.at<uchar>(Point(i,j));
            Scalar histIndex = SM.at<uchar>(Point(i,j));

            if(index.val[0]==255){
                whiteHistogram[(int)histIndex[0]]+=1;
                this->whiteHistogram[(int)histIndex[0]]+=1; // zsumowac bialy histogram
                whiteHistogramPixels+=1;
                this->whiteHistogramPixels+=1;
            }
            else if(index.val[0]==0){
                blackHistogram[(int)histIndex[0]]+=1;
                this->blackHistogram[(int)histIndex[0]]+=1;
                blackHistogramPixels+=1;
                this->blackHistogramPixels+=1;
            }
        }
    }

    double x = 0.0;
    double y = 0.0;
    double oldX = 0.0;
    double oldY = (float)AUCplot.rows;

    double AUCvalue = 0.0;
    double a = 0.0;
    double b = 0.0;
    double h = 0.0;

    for (int i=255;i>=0;i--){
        if(i<255) {
            oldX = oldX + x;
            oldY = oldY - y;
        }
        //cout << whiteHistogramPixels << " " << whiteHistogram[i] << endl;
        if (whiteHistogramPixels != 0) y = (double)whiteHistogram[i]/(double)whiteHistogramPixels*AUCplot.rows;
        else y = 0.0;
        if (blackHistogramPixels != 0) x = (double)blackHistogram[i]/(double)blackHistogramPixels*AUCplot.cols;
        else x = 0.0;
        line(AUCplot,Point(oldX,oldY),Point(oldX+x,oldY-y),Scalar(255,255,255),2);

        double normAUC = 1; //normalizeVal(AUCplot.rows,0,500);
        double normOldY = normalizeVal(oldY,0,500);
        double normY = normalizeVal(y,0,500);
        double normX = normalizeVal(x,0,500);

        a = normAUC - normOldY;
        b = normAUC - (normOldY - normY);
        h = normX;

        AUCvalue += ((a+b)*h)/2.0;
    }
    //string imgName = "./AUC/AUC_" + to_string(frameNumber) + ".jpg";
    //imwrite(imgName, AUCplot);

    //cout << "AUC for frame: " << frameNumber << "= " << AUCvalue << endl;
}

double Metric::getGlobalAUC(int numberOfFrames){
    return this->globalAUCvalue;
}

void Metric::drawGlobalAUCplot(){
    Mat AUCplot = Mat::zeros(500, 500, CV_32FC1);
    double x = 0.0;
    double y = 0.0;
    double oldX = 0.0;
    double oldY = (float)AUCplot.rows;

    double a = 0.0;
    double b = 0.0;
    double h = 0.0;

    this->globalAUCvalue = 0;


    for (int i=255;i>=0;i--){
        if(i<255) {
            oldX = oldX + x;
            oldY = oldY - y;
        }

        if (this->whiteHistogramPixels != 0) y = (double)this->whiteHistogram[i]/(double)this->whiteHistogramPixels*AUCplot.rows;
        else y = 0.0;
        if (this->blackHistogramPixels != 0) x = (double)this->blackHistogram[i]/(double)this->blackHistogramPixels*AUCplot.cols;
        else x = 0.0;

        line(AUCplot,Point(oldX,oldY),Point(oldX+x,oldY-y),Scalar(255,255,255),2);

        double normAUC = 1; //normalizeVal(AUCplot.rows,0,500);
        double normOldY = normalizeVal(oldY,0,500);
        double normY = normalizeVal(y,0,500);
        double normX = normalizeVal(x,0,500);

        a = normAUC - normOldY;
        b = normAUC - (normOldY - normY);
        h = normX;

        this->globalAUCvalue += ((a+b)*h)/2.0;
    }

    cout << "Writing global AUC..." << endl;
    string imgName = "./AUC/globalAUC.jpg";
    imwrite(imgName, AUCplot);
}

void Metric::saveAUC(){
    std::ofstream outfile ("./AUC/globalAUCvalue.txt");
    string text = to_string(this->globalAUCvalue);
    outfile << text << endl;
    outfile.close();
}

double Metric::getKLD(const Mat& a, const Mat& b){
    Scalar intensityA;
    Scalar intensityB;
    double iA, iB; // normalized intesity [0,1]
    double result = 0;
    for (int i=0; i<a.cols; i++){
        for(int j=0; j<a.rows; j++){
            intensityA = a.at<uchar>(Point(i, j));
            intensityB = b.at<uchar>(Point(i, j));
            iA = (double)intensityA[0]/255.0;
            iB = (double)intensityB[0]/255.0;
            //cout << iA << " " << iB << endl;
            if (iA!=0 && iB!=0) result += iA*(log10(iA) - log10(iB));
        }
    }
    return result/(a.cols*a.rows);
}

double Metric::getJ(const Mat& a, const Mat& b){
    Scalar intensityA;
    Scalar intensityB;
    double iA, iB; // normalized intesity [0,1]
    double J = 0;
    for (int i=0; i<a.cols; i++){
        for(int j=0; j<a.rows; j++){
            intensityA = a.at<uchar>(Point(i, j));
            intensityB = b.at<uchar>(Point(i, j));
            iA = intensityA[0]/255.0;
            iB = intensityB[0]/255.0;
            J += (iA*iB) / (iA + iB - iA*iB);
        }
    }
    return J/(a.cols*a.rows);
}

double Metric::getMSE(const Mat& a, const Mat& b){
    Scalar intensityA;
    Scalar intensityB;
    double iA, iB; // normalized intesity [0,1]
    double result = 0.0;
    for (int i=0; i<a.cols; i++){
        for(int j=0; j<a.rows; j++){
            intensityA = a.at<uchar>(Point(i, j));
            intensityB = b.at<uchar>(Point(i, j));
            iA = (double)intensityA[0]/255.0;
            iB = (double)intensityB[0]/255.0;
            result += (iA-iB)*(iA-iB);
        }
    }
    return result/(a.cols*a.rows);
}

double Metric::getScalar(const Mat& a, const Mat& b){
    Scalar intensityA;
    Scalar intensityB;
    double iA, iB; // normalized intesity [0,1]
    double S = 0;
    for (int i=0; i<a.cols; i++){
        for(int j=0; j<a.rows; j++){
            intensityA = a.at<uchar>(Point(i, j));
            intensityB = b.at<uchar>(Point(i, j));
            iA = intensityA[0]/255.0;
            iB = intensityB[0]/255.0;
            S += iA*iB;
        }
    }
    return S;
}
